@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,10 +33,11 @@
 #pragma once
 
 #include "../Core/DicomNetworking/IMoveRequestHandler.h"
-#include "ServerContext.h"
 
 namespace Orthanc
 {
+  class ServerContext;
+  
   class OrthancMoveRequestHandler : public IMoveRequestHandler
   {
   private:
